/*	File:	util.c
	Author:	Alex Fedosov, Jeff Malch
	Date:	January 31, 2001
	Description:	helpful utility functions
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "common/errors.h"
#include "common/object.h"

/* this function translates given (x,y) change in coordinates to
   a uniform direction and returns it */
int dmz_delta2dir(int x, int y)
{
	int d = 0;

	if (x > 0)
	{
		if(y > 0)
		{
			d = SOUTHEAST;
		}

		else if(y < 0)
		{
			d = NORTHEAST;
		}

		else d = EAST;
	}

	else if(x < 0)
	{
		if(y > 0)
		{
			d = SOUTHWEST;
		}
		else if (y < 0)
		{
			d = NORTHWEST;
		}
		else d = WEST;
	}
	else
	{
		if (y > 0)
		{
			d = SOUTH;
		}
		
		else if (y < 0)
		{
			d = NORTH;
		}
	}

	return d;
}

/* this function translates a given direction into appropriate change
   in (x,y) coordinates and adjusts them appropriately */
int dmz_dir2delta(int dir, WORD16 *x, WORD16 *y)
{
	if (x == NULL || y == NULL)
		return DMZ_BAD_PARAM;

	switch (dir)
	{
	case NORTH:
		(*y)--;
		break;
	case SOUTH:
		(*y)++;
		break;
	case EAST:
		(*x)++;
		break;
	case WEST:
		(*x)--;
		break;
	case SOUTHWEST:
		(*y)++;
		(*x)--;
		break;
	case SOUTHEAST:
		(*y)++;
		(*x)++;
		break;
	case NORTHWEST:
		(*y)--;
		(*x)--;
		break;
	case NORTHEAST:
		(*y)--;
		(*x)++;
		break;
	default:
		break;
	}

	return DMZ_SUCCESS;
}

int dmz_scale_delta(int dir, int *x, int *y)
{
	if (x == NULL || y == NULL)
		return DMZ_BAD_PARAM;

	switch (dir)
	{
	case NORTH:
		(*y)--;
		break;
	case SOUTH:
		(*y)++;
		break;
	case EAST:
		(*x)++;
		break;
	case WEST:
		(*x)--;
		break;
	case SOUTHWEST:
		(*y)++;
		(*x)--;
		break;
	case SOUTHEAST:
		(*y)++;
		(*x)++;
		break;
	case NORTHWEST:
		(*y)--;
		(*x)--;
		break;
	case NORTHEAST:
		(*y)--;
		(*x)++;
		break;
	default:
		break;
	}

	return DMZ_SUCCESS;
}

int strtoupr(char *string)
{
	int len = strlen(string) + 1;
	int i;

	for (i = 0; i < len; i++)
	{
		if (isalpha(string[i]))
			string[i] = toupper(string[i]);
	}

	return 0;
}