/*	File:	client.h
	Author:	Alex Fedosov
	Date:	December 12, 2000
	Description:	main client include file
*/

#ifndef __CLIENT_H__
#define __CLIENT_H__

#include "common/const.h"
#include "common/types.h"
#include "common/errors.h"
#include "common/map.h"
#include "common/object.h"
#include "common/msg.h"
#include "common/init.h"
#include "common/move.h"
#include "common/attack.h"
#include "common/items.h"
#include "common/upgrade.h"
#include "common/util.h"

#include "sys/linklist.h"

#define DMZ_CLIENT_VERSION 0x1201
#define DELTAX 20
#define DELTAY 19
#define MAX_FRAMES 4
#define MAXCHATBUF 35
#define CHAT_TIMER 10
#define MAXKBKEYS 256
#define VIRTUAL_W 928
#define VIRTUAL_H 864
#define PHYSICAL_W 640
#define PHYSICAL_H 600

int GameProcess (void);
void ProcessUnits(void);
void ProcessMessages(void);
void ProcessLocalInput(void);
int dmz_client_init(void* hwnd);
void dmz_client_kill(void);
int dmz_client_restart(void);
int dmz_connect_server(struct dmz_message *m);
int dmz_get_keyboard_input(int *dx, int *dy);
int do_move(int directionX, int directionY);
int do_attack(WORD16 player, WORD16 method, WORD16 dir);
void do_chat(char *buffer);
void set_screen_pos(unsigned int cx, unsigned int cy);
int dmz_mouse2delta(int mouse_x, int mouse_y, int *dx, int *dy);
void DrawStuff(int startW, int startH, int numW, int numH, int pxoff, int pyoff);

/* global variables */
GLOBAL int connid;					/* connection id */
GLOBAL WORD16 my_id;				/* player id */
GLOBAL char server_ip[16];			/* server address */
GLOBAL char player_name[16];		/* player name */

/* flags */
GLOBAL int chat_mode;				/* chat mode enabled */
GLOBAL int inv_mode;				/* inventory display */
GLOBAL int active;					/* game is active (running) */
GLOBAL int show_help;				/* toggle help screen */
GLOBAL int show_names;				/* toggle name labels */
GLOBAL int show_scores;				/* toggle scores display */
GLOBAL int show_exit;				/* toggle exit dialog */
GLOBAL int exit_game;				/* exit game flag */
GLOBAL int single_game;				/* single player game */

/* miscellaneous */
GLOBAL int sgdwFrame;				/* frame count */
GLOBAL int pixx;					/* screen position */
GLOBAL int pixy;
GLOBAL char chat_buf[MAXCHATBUF];	/* chat buffer */
GLOBAL char player_bitmaps[MAXCOLORS * MAXUPGRADES][64];
GLOBAL char buf[256];
GLOBAL char error[256];

/* image handles */
GLOBAL int transparent;				/* transparent color */
GLOBAL int virbuffer;				/* virtual buffer for clipping */
GLOBAL int frame;					/* game frame */
GLOBAL int progress;				/* progress indicator */
GLOBAL int splash, splash2;			/* splash screens */
GLOBAL int exitpic;					/* exit confirmation */
GLOBAL int portrait;				/* character portrait */
GLOBAL int alphabet, numbers;		/* font */
GLOBAL int inv, chatbox;			/* inventory & chatbox */
GLOBAL int missile, hth;			/* attack animation */
GLOBAL int help;					/* help screen */
GLOBAL int radarmap;				/* radar */
GLOBAL int lb, ab;					/* life and ammo status */
GLOBAL int bootybit;				/* booty upgrade */
GLOBAL WORD16 my_bitmap;			/* player's bitmaps */
GLOBAL WORD16 dead_player;
GLOBAL int lilalpha;		/* small font for labels */
GLOBAL int health, healthback, nameback;
GLOBAL int radar_player_color, radar_enemy_color,
			radar_item_color, radar_object_color;

/* sound handles */
GLOBAL int snd_bg;

/* message and chat queues */
GLOBAL LINK_LIST *incomingQ;
GLOBAL LINK_LIST *chatlist;

struct keyboardState
{
	int down;
};

GLOBAL struct keyboardState kb[MAXKBKEYS];

#endif
