/*	File:	dmzcore.h 
	Author:	Alex Fedosov
	Date:	September 21, 2000
	Modified: October 9, 2000
	Modified: November 9, 2000 -- combined headers into one file
	Description:	DMZ core API
*/

#ifndef __DMZ_CORE_H__
#define __DMZ_CORE_H__

#ifdef WIN32
	#ifdef __cplusplus
		#ifdef DMZCORE_EXPORTS
			#define DMZ_CORE_API extern "C" __declspec(dllexport)
		#else
			#define DMZ_CORE_API extern "C" __declspec(dllimport)
		#endif
	#else
		#ifdef DMZCORE_EXPORTS
			#define DMZ_CORE_API __declspec(dllexport)
		#else
			#define DMZ_CORE_API __declspec(dllimport)
		#endif
	#endif	/* __cplusplus */
#else
	#define DMZ_CORE_API
#endif

/*
DMZ_CORE_API size_t dmz_readfile(char *filename, char **data);
DMZ_CORE_API size_t dmz_writefile(char *filename, char *fileptr, size_t size);
*/

DMZ_CORE_API void dmz_logstd(char *msg);
DMZ_CORE_API void dmz_logerr(char *msg);
DMZ_CORE_API void dmz_die(char *err);
DMZ_CORE_API int dmz_set_config(const char *keyname, const char *option,
				   void *buffer, int size, int type);
DMZ_CORE_API int dmz_get_config(const char *keyname, const char *option,
				   void *buffer, int size, int type);
DMZ_CORE_API int dmz_core_get_last_error(void);

/* DMZ_CORE_API void* dmz_malloc(size_t size);
DMZ_CORE_API void dmz_free(void *ptr, size_t size);
*/

#define dmz_malloc(s) malloc(s)
#define dmz_free(p, s) free(p)

#ifndef NDEBUG
	/* Windows debugging support */
	#ifdef WIN32
		#include <windows.h>
		#define dmz_debug(dbgmsg, hwnd)\
			MessageBox(hwnd, dbgmsg, "Debug Message", MB_OK | MB_ICONERROR);
	/* ANSI C debugging support */
	#else
		#define dmz_debug(dbgmsg, hwnd)\
			printf(dbgmsg);\
			fflush(stdout);
	#endif	/* WIN32 */

	DMZ_CORE_API void dmz_assert_func(int x, char *file, int line);
	#define dmz_assert(x) dmz_assert_func((int) x, __FILE__, __LINE__)

#else
	/* disable debug support */
	#define dmz_debug(dbgmsg, hwnd) ;
	#define dmz_assert(x) ;
#endif	/* NDEBUG */

typedef enum
{
	DMZ_CONFIG_INT,
	DMZ_CONFIG_STRING
} DMZ_CONFIG_TYPES;

/* error codes */
typedef enum {
	DMZ_CORE_SUCCESS		=  0,	/* no error */
	DMZ_CORE_INVALID_PARAM	= -1,	/* invalid parameters specified */
	DMZ_CORE_CONFIG_ERR		= -2	/* config error from provider */
} DMZ_CORE_ERROR;

#endif
