/*	File:	linklist.h
	Author:	Alex Fedosov
	Date:	February 2, 2001
	Description:	Generic linked list interface
*/

#ifndef __LINKLIST_H__
#define __LINKLIST_H__

typedef struct dmz_list_elem
{
	struct dmz_list_elem *next;
	void *data;
	int reserved;
} LIST_NODE;

typedef struct dmz_linked_list
{
	struct dmz_list_elem *head;
	struct dmz_list_elem *tail;
	struct dmz_list_elem *pool;
	int numelems;
	int elemsize;
	int cur_num;
} LINK_LIST;

/* initializes the linked list */
LINK_LIST* LL_init(int numelems, int elemsize);

/* destroys the linked list */
void LL_destroy(LINK_LIST *myLL);

/* adds an item to the end of the list */
int LL_add(LINK_LIST *myLL, void *src, int reserved);

/* copies an item from the list without removing it into memory
   pointed to by elem. user-defined field is copied into reserved.
   if location is 0, head element is copied. if location is negative,
   tail element is copied. if location is positive, "next" element is
   copied. "next" is kept track of internally.
*/
int LL_get(LINK_LIST *myLL, void *dest, int location);

/* get an element from the head of the list */
int LL_remove(LINK_LIST *myLL, void *dest);

/* returns number of elements currently on the list */
int LL_num_elements(LINK_LIST *myLL);

#endif
