/*	File:	client.c
	Author:	Alex Fedosov, Jeff Malch
	Date:	February 1, 2001
	Description:	client engine -- main game logic
*/

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "client.h"
#include "input.h"

#include "dmz_net.h"
#include "dmzcore.h"
#include "dmzgraph.h"
#include "dmz_snd.h"

#include "sys/thread.h"

char server_ip[16];
char player_name[16];

int portrait, alphabet, numbers, inv, help, chatbox, exitpic;	/* bitmaps */
int virbuffer, transparent;
int splash, splash2;
int nameback, health, healthback, lilalpha;
int radarmap;
int lb, ab, bootybit;
int frame, progress;
int missile, explosion, hth;

int snd_bump, snd_bg;				/* sounds */
int snd_fire, snd_empty, snd_ammo, snd_armor;
int snd_ouch, snd_nuke, snd_join, snd_hth;
int snd_dead, snd_dead1;

int connid;

int active = 0, sgdwFrame = 0, exit_game = 0;
int pixx = 0, pixy = 0;

int show_help = 0, show_names = 1, show_scores = 0, show_exit = 0;

WORD16 my_id = 0;
WORD16 my_bitmap;
WORD16 dead_player;

LINK_LIST *incomingQ;
LINK_LIST *chatlist;

char player_bitmaps[MAXCOLORS * MAXUPGRADES][64] =
{
	 "data/Robots/RobotA00.bmp",
	 "data/Robots/RobotB00.bmp",
	 "data/Robots/RobotC00.bmp",
	 "data/Robots/RobotD00.bmp",
 	 "data/Robots/RobotA01.bmp",
	 "data/Robots/RobotB01.bmp",
	 "data/Robots/RobotC01.bmp",
	 "data/Robots/RobotD01.bmp",
 	 "data/Robots/RobotA02.bmp",
	 "data/Robots/RobotB02.bmp",
	 "data/Robots/RobotC02.bmp",
	 "data/Robots/RobotD02.bmp",
 	 "data/Robots/RobotA03.bmp",
	 "data/Robots/RobotB03.bmp",
	 "data/Robots/RobotC03.bmp",
	 "data/Robots/RobotD03.bmp",
};
char player_portraits[MAXCOLORS][64] =
{
	"data/Portraits/Mechanic_sm.bmp",
	"data/Portraits/Comander_sm.bmp",
	"data/Portraits/Alien_sm.bmp",
	"data/Portraits/Alien2_sm.bmp",
};

char buf[256];
GLOBAL HWND hWnd;
time_t start_time, finish_time;

void DrawProgressIndicator(int percent, int flag);

int GameProcess (void)
{
	DWORD   dwTick = GetTickCount();
	static DWORD sgdwNextFrame = 0;
	int start_x, start_y, offset_x, offset_y;

	/* get local input */
	dmz_read_keyboard();

	/* check for frame advance */
	if (dwTick >= sgdwNextFrame)
	{
		sgdwNextFrame = dwTick + 40;	/* Set next frame */
		sgdwFrame++;

		/* process units no more than 25 times per second */
		ProcessUnits();

		start_x = pixx / PIXELS_PER_CELL;
		start_y = pixy / PIXELS_PER_CELL;
		offset_x = pixx % PIXELS_PER_CELL;
		offset_y = pixy % PIXELS_PER_CELL;
		DrawStuff(start_x, start_y, start_x + DELTAX, start_y + DELTAY, offset_x, offset_y);
	}

	return !exit_game;
}

void ReceiveMessages(void *arg)
{
	static struct dmz_message m;
	static int bytes_reqd = sizeof(struct dmz_message);
	static int bytes_recv = 0;
	char *dest = (char *) &m;
	int rv;

	dmz_net_chmode(connid, BLOCKING);

	while (1)
	{
		dest += bytes_recv;
		rv = dmz_recv_msg(dest, bytes_reqd, connid);

		/* full message received */
		if (rv == bytes_reqd)
		{
			LL_add(incomingQ, &m, 1);
			bytes_recv = 0;
			bytes_reqd = sizeof(struct dmz_message);
			dest = (char *) &m;
		}

		else if (rv < 0) /* error occurred */
		{
			sprintf(buf, "[client:client:GameProcess] network error %d, bailing out", rv);
			dmz_logerr(buf);

			DestroyWindow(hWnd);
			dmz_die("connection to server lost!");
			break;
		}

		else	/* partial message received */
		{
			bytes_recv = rv;
			bytes_reqd -= rv;
		}
	}
}

void ProcessUnits(void)
{
	WORD16 i;

	/* go through all the objects and animate as needed */
	for (i = 0; i < MAXOBJECTS; i++)
	{
		struct dmz_object *o = dmz_objects[i];
		if (dmz_objects[i] == NULL) continue;

		switch (dmz_objects[i]->objtype)
		{
		case MONSTER:
		case PLAYER:
			/* animate once every four frames */
			if (dmz_objects[i]->status == MOVING && (sgdwFrame % 4) == 0)
			{
				dmz_objects[i]->u.player.frame++;
				if (dmz_objects[i]->u.player.frame >= MAX_FRAMES)
					dmz_objects[i]->u.player.frame = 0;

				/* reset status */
				dmz_objects[my_id]->status = IDLE;
			}
			break;

		case ITEM:
			if ( (sgdwFrame % 4) == 0)
			{
				dmz_objects[i]->u.item.item_frame++;
				if (dmz_objects[i]->u.item.item_frame >= MAX_FRAMES)
					dmz_objects[i]->u.item.item_frame = 0;
			}
			break;

		case MISSILE:
			{
				/* destroy expired missiles, explosion and all */
				if (dmz_objects[i]->u.missile.range == 0)
				{
					dmz_status_update(i, DEAD);
					dmz_objects[i]->u.missile.exp_frame++;

					dmz_objects[i]->width = 48;
					dmz_objects[i]->height = 48;

					if(!(--dmz_objects[i]->u.missile.exp_ttl))
						dmz_delete_object(i);
				}

				/* out of range */
				else if (dmz_objects[i]->x <= 0 || dmz_objects[i]->x >= map_width * PIXELS_PER_CELL ||
					dmz_objects[i]->y <= 0 || dmz_objects[i]->y >= map_height * PIXELS_PER_CELL)
				{
					/* destroy missile */
					dmz_delete_object(i);
				}

				else
				{
					WORD16 target = 0;
					int dx = 0, dy = 0;

					/* decrease TTL */
					dmz_objects[i]->u.missile.range--;

					/* update location */
					dmz_scale_delta(o->u.missile.direction, &dx, &dy);
					dx *= o->velocity;
					dy *= o->velocity;

					target = DetectCollision(o->objid);

					if (target != 0)
					{
						/* if missile collision, delete both */
						if (dmz_objects[target]->objtype == MISSILE)
							dmz_delete_object(target);
					
						dmz_status_update(i, DEAD);

						/* expire the missile */
						dmz_objects[i]->u.missile.range = 0;
						
						/* advance the explosion along the trajectory */
						o->x += dx;
						o->y += dy;
					}

					else
					{
						/* put it in the new location */
						o->x += dx;
						o->y += dy;
					}
				}
			}
			break;

		default:	/* don't animate anything else */
			break;
		}
	}

	/* process any received network messages */
	ProcessMessages();

	/* finally process user input from the console */
	ProcessLocalInput();
}

void ProcessMessages(void)
{
	struct dmz_message m;
	static int q_index = 0;

	while(LL_remove(incomingQ, &m))
	{
		switch (m.type)
		{
		case BEGIN:
		{
			dmz_logstd("BEGIN message");
			/* only players should be sending BEGIN messages */
			dmz_new_object(m.objid, PLAYER, m.u.begin.init_x, m.u.begin.init_y);
			memcpy(dmz_objects[m.objid]->name, m.u.begin.name, NAMELEN);
			dmz_objects[m.objid]->bitidx = m.u.begin.bitmap;
			dmz_objects[m.objid]->deadbit = dead_player;
			dmz_objects[m.objid]->u.player.direction = m.u.begin.init_dir;

			sprintf(buf, "[client:ct:ProcessUnits] new player of id %d bitmap %d", m.objid, m.u.begin.bitmap);
			dmz_logstd(buf);

			if (m.objid == my_id)
			{
				/* player respawning */
				set_screen_pos(dmz_objects[my_id]->x, dmz_objects[my_id]->y);
			}

			dmz_snd_play(snd_join, 0);

			sprintf(buf, "%s JOINED THE GAME", dmz_objects[m.objid]->name);
			LL_add(chatlist, buf, time(NULL));
		}
		break;

		case MOVE:
			if (m.objid == my_id) break;

			dmz_assert(m.objid);

			dmz_objects[m.objid]->u.player.direction = m.u.move.direction;
			dmz_objects[m.objid]->x = m.u.move.new_x;
			dmz_objects[m.objid]->y = m.u.move.new_y;

			break;

		case ATTACK:
			if (m.objid == my_id) break;
			dmz_assert(m.objid);
			
			if (m.u.attack.method == PROJECTILE)
			{
				MissileAttack(m.objid, m.u.attack.direction);
			}

			else if (m.u.attack.method == HTH)
				HTHAttack(m.objid, dmz_objects[m.objid]->u.player.direction);

			break;

		case CHAT:
			/* add to chat queue */
			sprintf(buf, "CHAT received from %d: %s", m.objid, m.u.chat.text);
			dmz_logstd(buf);
			sprintf(buf, "%s: %s", dmz_objects[m.objid]->name, m.u.chat.text);
			LL_add(chatlist, buf, time(NULL));
			
			break;

		case OBJECT:	/* received an object, add it to the object array */
		{
			int new_objid = m.u.object.obj.objid;

			sprintf(buf, "OBJECT message objid %d", m.objid);
			dmz_logstd(buf);

			if (dmz_objects[new_objid] == NULL)
				dmz_objects[new_objid] = (struct dmz_object *) malloc(sizeof(struct dmz_object));
			
			memcpy(dmz_objects[new_objid], &m.u.object.obj, sizeof(struct dmz_object));

			break;
		}

		case UPDATE:	/* update somebody's stats TODO for other object types */
		{
			short life = m.u.update.life;
			short score = m.u.update.score;
			short status = m.u.update.status;
			short ammo = m.u.update.ammo;
			short frags = m.u.update.frags;

			sprintf(buf, "[client:dmz_process_msg] UPDATE object %d: life: %d, ammo: %d, frags: %d, score: %d, status: %d",
				m.objid, life, ammo, frags, score, status);
			dmz_logstd(buf);

			/* life update */
			if (life >= 0)
			{
				/* determine whether to play any sounds */
				if (m.objid == my_id)
				{
					if (life > dmz_objects[m.objid]->u.player.life)
						dmz_snd_play(snd_armor, 0);

					else if (life < dmz_objects[m.objid]->u.player.life)
						dmz_snd_play(snd_ouch, 0);

				}

				dmz_objects[m.objid]->u.player.life = (WORD16) life;

				if (life == 0)	/* somebody died */
				{
					char buffer[MAXCHATBUF + 1];
					sprintf(buffer, "%s GOT AXED", dmz_objects[m.objid]->name);

					LL_add(chatlist, buffer, time(NULL));

					dmz_status_update(m.objid, DEAD);

					dmz_snd_play(snd_dead, 0);
				}
			}

			/* ammo update */
			if (ammo >= 0)
			{
				/* determine whether to play any sounds */
				if (m.objid == my_id)
				{
					if (ammo > dmz_objects[m.objid]->u.player.ammo)
						dmz_snd_play(snd_ammo, 0);
				}

				dmz_objects[m.objid]->u.player.ammo = (WORD16) ammo;
			}

			/* score update */
			if (score >= 0)
			{
				dmz_objects[m.objid]->score = (WORD16) score;
				/* upgrade if needed */
				if (dmz_objects[m.objid]->objtype == PLAYER)
					UpgradePlayer(m.objid);
			}

			/* frags update */
			if (frags >= 0)
			{
				dmz_objects[m.objid]->u.player.frags = (WORD16) frags;
			}

			if (status >= 0) dmz_objects[m.objid]->status = (WORD16) status;

			break;
		}

		case DEATH:
			{
				char buffer[MAXCHATBUF + 1];
				sprintf(buffer, "%s GOT DESTROYED", dmz_objects[m.objid]->name);

				dmz_snd_play(snd_nuke, 0);

				LL_add(chatlist, buffer, time(NULL));
			}

			sprintf(buf, "DEATH message for %s", dmz_objects[m.objid]->name);
			dmz_logstd(buf);
			dmz_status_update(m.objid, DEAD);

			break;

		case END:
			if (dmz_objects[m.objid]->status == DEAD)
				dmz_delete_object(m.objid);
			dmz_status_update(m.objid, SLEEPING);

			sprintf(buf, "%s LEFT THE GAME", dmz_objects[m.objid]->name);
			LL_add(chatlist, buf, time(NULL));
			break;

		default:
			break;
		}
	}
}

int dmz_client_init(void* hwnd)
{
	int err, i;
	struct dmz_message m;

	dmz_logstd("[client:client:dmz_client_init] starting intialization");

	incomingQ = LL_init(128, sizeof(struct dmz_message));
	chatlist = LL_init(8, MAXCHATBUF);

	/* initialize graphics library */
	err = dmz_graph_init(hwnd);
	if (err != DMZ_GRAPH_SUCCESS)
	{
		sprintf(buf, "[client:client:init] graph_init() returned %d", err);
		dmz_logerr(buf);
		sprintf(error, "Error initializing graphics library");
		return DMZ_INIT_ERROR;
	}

	/* sets display mode */
	err = dmz_graph_set_mode(800, 600, 16);
	if (err != DMZ_GRAPH_SUCCESS)
	{
		sprintf(buf, "[client:client:init] set_mode() returned %d", err);
		dmz_logerr(buf);
		sprintf(error, "Could not switch to requested video mode");
		return DMZ_INIT_ERROR;
	}

	/* figure out transparent color  and colors for the radar */
	transparent = dmz_graph_mapRGB(255, 0, 255);
	radar_player_color = dmz_graph_mapRGB(0, 0, 0);
	radar_enemy_color = dmz_graph_mapRGB(255, 0, 0);
	radar_item_color = dmz_graph_mapRGB(255, 255, 255);
	radar_object_color = dmz_graph_mapRGB(128, 0, 0);

	/* load player skins to avoid discrepancies */
	for (i = 0; i < MAXCOLORS * MAXUPGRADES; i++)
	{
		dmz_graph_load_image(player_bitmaps[i], NULL, NULL);
	}
	dead_player = dmz_graph_load_image("data/Robots/dead.bmp", NULL, NULL);
	bootybit = dmz_graph_load_image("data/Booty/booty.bmp", NULL, NULL);

	progress = dmz_graph_load_image("data/progress.bmp", NULL, NULL);

	/* show splash screen */
	splash = dmz_graph_load_image("data/wait.bmp", NULL, NULL);
	DrawProgressIndicator(0, 0);

	/* load and init data */
	err = dmz_data_init();
	if (err != DMZ_SUCCESS)
	{
		sprintf(buf, "[client:client:init] data_init() returned %d", err);
		dmz_logerr(buf);
		
		sprintf(error, "Error loading data files");
		return DMZ_INIT_ERROR;
	}

	/* first message must be BEGIN */
	memset(&m, 0, sizeof(struct dmz_message));
	m.type = BEGIN;
	m.objid = 0;
	m.u.begin.bitmap = my_bitmap;

	m.u.begin.init_dir = NORTH;
	m.u.begin.init_x = START_POS_X * PIXELS_PER_CELL;
	m.u.begin.init_y = START_POS_Y * PIXELS_PER_CELL;
	memcpy(m.u.begin.name, player_name, NAMELEN);

	splash2 = dmz_graph_load_image("data/wait2.bmp", NULL, NULL);
	DrawProgressIndicator(80, !single_game);

	/* connect to server */
	err = dmz_connect_server(&m);
	if (err != DMZ_SUCCESS)
	{
		sprintf(buf, "[client:client:init] dmz_connect_server() returned %d", err);
		dmz_logerr(buf);
		
		sprintf(error, "Error connecting to server");
		return DMZ_INIT_ERROR;
	}
	
	/* create player object */
	m.objid = my_id = dmz_new_object(m.objid, PLAYER, m.u.begin.init_x, m.u.begin.init_y);
	my_bitmap = dmz_objects[my_id]->bitidx = m.u.begin.bitmap;
	dmz_objects[my_id]->deadbit = dead_player;
	dmz_objects[my_id]->u.player.direction = m.u.begin.init_dir;
	memcpy(dmz_objects[my_id]->name, m.u.begin.name, NAMELEN);
	sprintf(buf, "[client:client:init] server gave us id %d", my_id);
	dmz_logstd(buf);

	/* center the screen around the character */
	pixx = m.u.begin.init_x - (DELTAX >> 1) * PIXELS_PER_CELL;
	pixy = m.u.begin.init_y - (DELTAY >> 1) * PIXELS_PER_CELL;

	/* create virtual buffer for clipping */
	virbuffer = dmz_graph_create_virtual(VIRTUAL_W, VIRTUAL_H);

	/* load interface bitmaps */
	portrait = dmz_graph_load_image(player_portraits[(my_bitmap - 1) % 4], NULL, NULL);
	missile = dmz_graph_load_image("data/Special/missile.bmp", NULL, NULL);
	hth = dmz_graph_load_image("data/Special/hth.bmp", NULL, NULL);
	explosion = dmz_graph_load_image("data/Special/explosion.bmp",NULL,NULL);
	alphabet = dmz_graph_load_image("data/alphabet.bmp", NULL, NULL);
	numbers = dmz_graph_load_image("data/numbers.bmp", NULL, NULL);
	inv = dmz_graph_load_image("data/inventory.bmp", NULL, NULL);
	help = dmz_graph_load_image("data/help.bmp", NULL, NULL);
	frame = dmz_graph_load_image("data/frame3a.bmp", NULL, NULL);
	chatbox = dmz_graph_load_image("data/chatbox.bmp", NULL, NULL);
	exitpic = dmz_graph_load_image("data/exit.bmp", NULL, NULL);
	radarmap = dmz_graph_load_image("dmz_map.bmp", NULL, NULL);

	/* for life & ammo status bars */
	lb = dmz_graph_load_image("data/Status/life_bar.bmp", NULL, NULL);
	ab = dmz_graph_load_image("data/Status/ammo_bar.bmp", NULL, NULL);

	/* for player name labels */
	nameback = dmz_graph_load_image("data/Status/nameback.bmp", NULL, NULL);
	health = dmz_graph_load_image("data/Status/health.bmp", NULL, NULL);
	healthback = dmz_graph_load_image("data/Status/blank.bmp", NULL, NULL);
	lilalpha = dmz_graph_load_image("data/Status/lilalpha.bmp", NULL, NULL);

	DrawProgressIndicator(90, !single_game);

	/* load sounds */
	dmz_snd_init(hwnd);
	snd_bump = dmz_snd_load("data/Sound/bump1.wav");
	snd_bg = dmz_snd_load("data/Sound/bg.wav");
	snd_fire = dmz_snd_load("data/Sound/blaster.wav");
	snd_empty = dmz_snd_load("data/Sound/empty.wav");
	snd_dead = dmz_snd_load("data/Sound/dead.wav");
	snd_dead1 = dmz_snd_load("data/Sound/dead1.wav");
	snd_ammo = dmz_snd_load("data/Sound/ammoup.wav");
	snd_armor = dmz_snd_load("data/Sound/armorup.wav");
	snd_ouch = dmz_snd_load("data/Sound/ouch.wav");
	snd_join = dmz_snd_load("data/Sound/join.wav");
	snd_nuke = dmz_snd_load("data/Sound/nuke.wav");
	snd_hth = dmz_snd_load("data/Sound/electro.wav");

	DrawProgressIndicator(100, !single_game);

	dmz_snd_play(snd_bg, DMZ_SND_LOOP);
	dmz_snd_set_volume (snd_bg, 65);

	dmz_snd_play(snd_join, 0);
	dmz_snd_set_volume (snd_join, 65);

	LL_add(chatlist, "PRESS F1 FOR HELP", time(NULL));

	active = 1;

	start_time = time(NULL);

	return DMZ_SUCCESS;
}

int dmz_connect_server(struct dmz_message *m)
{
	int player, rv;
	unsigned int h, tid;

	/* attempt to connect to server */
	sprintf(buf, "[client:dmz_connect_server] connecting to %s", server_ip);
	dmz_logstd(buf);

	sprintf(buf, "[client:dmz_connect_server] player name is %s", player_name);
	dmz_logstd(buf);

	dmz_net_init();

	connid = dmz_connect(server_ip, DMZPORT);
	if (connid < 0)
	{
		sprintf(buf, "dmz_connect failed, returned %d", connid);
		dmz_logerr(buf);
			
		sprintf(error, "Error connecting to server");
		return DMZ_NETWORK_ERROR;
	}

	/*dmz_net_chmode(connid, NONBLOCKING);*/

	sprintf(buf, "[client:dmz_connect_server] connected to server %s", server_ip);
	dmz_logstd(buf);

	/* attempt to retrieve player id */
	rv = dmz_get_config("DMZ", player_name, &player, sizeof(int), DMZ_CONFIG_INT);
	if (rv == DMZ_CORE_SUCCESS)
			m->objid = player;

	rv = dmz_send_msg(m, sizeof(struct dmz_message), connid);
	if (rv < 0)
	{
		sprintf(buf, "[client:dmz_connect_server] failed to establish connection with server: send error %d", rv);
		dmz_logerr(buf);
			
		sprintf(error, "Network Error: could not send data");
		return DMZ_NETWORK_ERROR;
	}

/*		while ((err = dmz_recv_msg(&m, sizeof(struct dmz_message), connid)) == DMZ_NET_TRY_AGAIN);

		if (err < 0)
		{
			sprintf(buf, "[client:client:init] failed to establish connection with server: receive error %d", err);
			dmz_logerr(buf);
			return DMZ_INIT_ERROR;
		}
*/
	/* create an I/O thread to receive messages */
	h = dmz_thread_create(ReceiveMessages, NULL, &tid);
	if (h == 0)
	{
		dmz_logerr("[client:dmz_client_init] failed to create thread!");
		sprintf(error, "Error creating I/O thread");
		return DMZ_NETWORK_ERROR;
	}
		
	/* wait for server's acknowledgements of our BEGIN message */
	start_time = time(NULL);
	while ((time(NULL) - start_time) < NETWORK_TIMEOUT)
	{
		int rv = LL_remove(incomingQ, m);

		if (rv != 0)
		{
			/* must be begin message with matching player name */
			// TODO currently no name check because of case problems
			if (m->type == BEGIN) //&& strcmp(player_name, m.u.begin.name) == 0)
			{
				dmz_logstd("client:client:dmz_client_init] server acknowledgement received");
				break;
			}

			else
				LL_add(incomingQ, m, 1);
		}
	}

	/* network timeout occurred */
	if ((time(NULL) - start_time) >= NETWORK_TIMEOUT)
	{
		dmz_logerr("[client:dmz_client_init] network timeout, no response from server");
		sprintf(error, "Network timeout");
		return DMZ_NETWORK_ERROR;
	}

	/* save settings */
	player = m->objid;
	dmz_set_config("DMZ", "Server Address", server_ip, 16, DMZ_CONFIG_STRING);
	dmz_set_config("DMZ", "Last Player", player_name, NAMELEN, DMZ_CONFIG_STRING);
	dmz_set_config("DMZ", player_name, &player, sizeof(int), DMZ_CONFIG_INT);

	return DMZ_SUCCESS;
}

void dmz_client_kill(void)
{
	struct dmz_message m;

	memset(&m, 0, sizeof(struct dmz_message));

	active = 0;

	finish_time = time(NULL);

	sprintf(buf, "Frame Rate: %.2f fps", (float) sgdwFrame / (float) (finish_time - start_time));
	dmz_logstd(buf);

	memset(&m, 0, sizeof(struct dmz_message));

	m.objid = my_id;
	m.type = END;

	dmz_send_msg(&m, sizeof(struct dmz_message), connid);

	dmz_snd_kill();
	dmz_object_destroy();
	dmz_logstd("[client:client:dmz_client_kill] object destruction");
	dmz_map_destroy();
	dmz_logstd("[client:client:dmz_client_kill] map destruction");
	dmz_snd_kill();
	dmz_logstd("[client:client:dmz_client_kill] sound shutdown");
	dmz_graph_kill();
	dmz_logstd("[client:client:dmz_client_kill] graphics shutdown");
	dmz_net_kill();
	dmz_logstd("[client:client:dmz_client_kill] network shutdown");
}

int dmz_client_restart(void)
{
	struct dmz_message m;
	int err;

	memset(&m, 0, sizeof(struct dmz_message));
	m.type = BEGIN;
	m.objid = my_id;
	m.u.begin.bitmap = my_bitmap;
	memcpy(m.u.begin.name, player_name, NAMELEN);
	
	err = dmz_send_msg(&m, sizeof(struct dmz_message), connid);

	if (err < 0)
	{
		sprintf(buf, "[client:client:init] failed to establish connection with server: send error %d", err);
		dmz_logerr(buf);
		return DMZ_INIT_ERROR;
	}

	return DMZ_SUCCESS;
}

void set_screen_pos(unsigned int cx, unsigned int cy)
{
	/* only update screen if not beyond limit */
	int horz_limit = map_width * PIXELS_PER_CELL - PHYSICAL_W - 1;
	int vert_limit = map_height * PIXELS_PER_CELL - PHYSICAL_H - 1;
		
	pixx = cx - (DELTAX >> 1) * PIXELS_PER_CELL;
	if (pixx < 128) pixx = 128;
	else if (pixx > horz_limit) pixx = horz_limit;

	pixy = cy - (DELTAY >> 1) * PIXELS_PER_CELL;
	if (pixy < 128) pixy = 128;
	else if (pixy > vert_limit) pixy = vert_limit;
}
