/*	File:	draw.c
	Author:	Alex Fedosov, Jeff Malch
	Date:	February 1, 2001
	Description:	client engine -- drawing routines
*/

#include <stdio.h>
#include <time.h>

#include "client.h"
#include "draw.h"

#include "dmzcore.h"
#include "dmzgraph.h"

#define TRANSPARENT_COLOR transparent

int radar_player_color, radar_enemy_color, radar_item_color, radar_object_color;

void DrawStuff(int startW, int startH, int numW, int numH, int pxoff, int pyoff)
{
	int i, j, k;
	char chatbuffer[MAXCHATBUF];
	time_t tstamp, now;
	int chat_offset = 0, numchats = 0;
	int dstx = 0, dsty = 0;

	int x_adj = startW * PIXELS_PER_CELL - 128;
	int y_adj = startH * PIXELS_PER_CELL - 128;

	if (!active) return;

	/* draw the background map */
	for (i = startW - 4; i <= numW; i++)
	{
		for (j = startH - 4; j <= numH; j++)
		{
			dmz_assert(i >= 0);
			dmz_assert(j >= 0);

			k = dmz_world_map[i][j];

			dmz_assert(dmz_map_cells[k]);

			dmz_graph_virblit(k, virbuffer, 0, 0, 32, 32, dstx, dsty, -1);
			dsty += 32;
		}
		dstx += 32;
		dsty = 0;
	}

	{
		int rad_x = 0, rad_y = 0;

		rad_x = dmz_objects[my_id]->x / PIXELS_PER_CELL;
		rad_y = dmz_objects[my_id]->y / PIXELS_PER_CELL;

		rad_x -= 50;
		rad_y -= 50;

		if ((rad_x + 100) >= (int) map_width) rad_x = map_width - 100 - 1;
		else if (rad_x < 0) rad_x = 0;

		if ((rad_y + 100) >= (int) map_height) rad_y = map_height - 100 - 1;
		else if (rad_y < 0) rad_y = 0;

		dmz_graph_blit(radarmap, rad_x, rad_y, 100, 100, 669, 421, -1);
	}

	/* draw the objects */
	for (i = 0; i < MAXOBJECTS; i++)
	{
			if (dmz_objects[i] != NULL)
			{
				int bitID, d = 0, f = 0;
				int w = dmz_objects[i]->width;
				int h = dmz_objects[i]->height;

				int dstx = dmz_objects[i]->x - x_adj;
				int dsty = dmz_objects[i]->y - y_adj;

				/* don't draw invisible objects */
				if (dmz_objects[i]->status == INVISIBLE ||
					dmz_objects[i]->status == SLEEPING) continue;

				if (dstx > 0 && dstx < VIRTUAL_W && dsty > 0 && dsty < VIRTUAL_H)
				{
					bitID = dmz_objects[i]->bitidx;

					if (dmz_objects[i]->status == DEAD)
					{
						bitID = dmz_objects[i]->deadbit;
					}

					if (dmz_objects[i]->objtype == TERRAIN)
					{
						if (dmz_objects[i]->status != DEAD)
							DrawOtherRadarBlip(i, 3, 3, radar_object_color);
					}

					else if (dmz_objects[i]->objtype == PLAYER || dmz_objects[i]->objtype == MONSTER)
					{
						d = dmz_objects[i]->u.player.direction;
						f = dmz_objects[i]->u.player.frame;
						
						if (i == my_id)
							DrawPlayerRadarBlip(i, radar_player_color);
						else if (dmz_objects[i]->status != DEAD)
							DrawPlayerRadarBlip(i, radar_enemy_color);
						
						/* display character name if enabled */
						if (show_names && i != my_id)
						{
							int lilindex, e_life;
							int namelen = strlen(dmz_objects[i]->name);
							int n_dest;
							
							if (namelen > 12) namelen = 12;
							n_dest = (dstx + (dmz_objects[i]->width >> 1)) - (namelen << 2);

							dmz_graph_virblit(nameback, virbuffer, 0, 0, namelen * 8, 10, n_dest, dsty - 10, TRANSPARENT_COLOR);
							for (lilindex = 0; lilindex < namelen; lilindex++)
								dmz_graph_virblit(lilalpha, virbuffer, ((dmz_objects[i]->name[lilindex]) - 'A') * 8, 0,
									8, 8, (n_dest + lilindex * 8), dsty - 9, TRANSPARENT_COLOR);
							
							e_life = dmz_objects[i]->u.player.life;
							dmz_graph_virblit(health, virbuffer, 0, 0, e_life >> 1, 5, n_dest, dsty - 16, -1);
							dmz_graph_virblit(healthback, virbuffer, 0, 0, 50, 5, n_dest, dsty - 16, TRANSPARENT_COLOR);
						}
					}

					else if (dmz_objects[i]->objtype == ITEM)
					{
						f = dmz_objects[i]->u.item.item_frame;
						DrawOtherRadarBlip(i, 2, 2, radar_item_color);
					}

					else if (dmz_objects[i]->objtype == MISSILE)
					{
						if (dmz_objects[i]->status == DEAD)
						{
							f = dmz_objects[i]->u.missile.exp_frame;

							/* adjust explosion position */
							dstx -= (DEFAULT_WIDTH >> 2);
							dsty -= (DEFAULT_HEIGHT >> 1);

							/*
							w = 48;	// TODO: hard coding sucks!
							h = 48;
							*/
						}
					}

					dmz_graph_virblit(bitID, virbuffer, f * w, d * h, w, h, dstx, dsty, TRANSPARENT_COLOR);
				}

				/* object not visible on the screen,
				   but may appear on the radar */
				else if ((dmz_objects[i]->objtype == PLAYER ||
						 dmz_objects[i]->objtype == MONSTER) &&
						 dmz_objects[i]->status != DEAD)
					DrawPlayerRadarBlip(i, radar_enemy_color);

				else if (dmz_objects[i]->objtype == TERRAIN &&
					dmz_objects[i]->status != DEAD)
					DrawOtherRadarBlip(i, 3, 3, radar_object_color);
				else if (dmz_objects[i]->objtype == ITEM)
					DrawOtherRadarBlip(i, 2, 2, radar_item_color);
			}
	}

	/* transfer the virtual buffer to physical video memory */
	dmz_graph_virblit(virbuffer, 0, 128 + pxoff, 128 + pyoff, PHYSICAL_W, PHYSICAL_H, 0, 0, -1);

	numchats = LL_num_elements(chatlist);

	while (numchats > 0)
	{
		/* only get current time once! */
		if (chat_offset == 0)
			now = time(NULL);

		chat_offset++;

		/* get something from chat queue */
		tstamp = LL_remove(chatlist, chatbuffer);

		DrawString(chatbuffer, CHAT_X, CHAT_Y * chat_offset);

		/* check for expiration... add back to queue if needed */
		if ((tstamp + CHAT_TIMER) > now)
			LL_add(chatlist, chatbuffer, tstamp);

		numchats--;
	}

	if (inv_mode)
	{
		int item;
		int count = 0;

		dmz_graph_virblit(inv, 0, 0, 0, 300, 100, DRAW_INV_X, DRAW_INV_Y, -1);

		for (i = 0; i < MAX_INV; i++)
		{
			item = dmz_objects[my_id]->inv[i];

			if (item != 0)
			{
				dmz_assert(dmz_objects[item]);
				dmz_graph_virblit(dmz_objects[item]->bitidx, 0, 0, 0, 32, 32,
					DRAW_INV_X + 12 + (count++ * 32), DRAW_INV_Y + 12, TRANSPARENT_COLOR);
			}
		}
	}

	dmz_graph_virblit(frame, 0, 0, 0, 160, 600, 640, 0, TRANSPARENT_COLOR);
	dmz_graph_virblit(portrait, 0, 0, 0, 100, 100, 671, 46, -1);
	
	if (chat_mode == 1)
	{
		dmz_graph_blit(chatbox, 0, 0, 580, 43, 40, 520, -1);

		DrawString(chat_buf, INPUT_X, INPUT_Y);
	}

	/* display life. score, ammo */

	k = dmz_objects[my_id]->u.player.life;
	dmz_graph_virblit(lb, 0, 0, 0, k, 16, 669, 221, -1);

	k = dmz_objects[my_id]->u.player.ammo;
	dmz_graph_virblit(ab, 0, 0, 0, k, 16, 669, 297, -1);

	sprintf(chatbuffer, "%d", dmz_objects[my_id]->u.player.frags);
	DrawString(chatbuffer, 705, 370);

	if (show_help)
		dmz_graph_blit(help, 0, 0, 400, 320, HELP_X, HELP_Y, TRANSPARENT_COLOR);

	if (show_scores)
		DrawCurrentScores();

	if (show_exit)
		dmz_graph_blit(exitpic, 0, 0, 200, 85, 220, 250, -1);

	dmz_graph_flip();
}

void DrawCurrentScores(void)
{
	static int last = 0;
	int i;
	int players = 0;

	for (i = last; i < MAXOBJECTS; i++)
	{
		if (dmz_objects[i] == NULL) continue;
		if (dmz_objects[i]->objtype != PLAYER) continue;
		if (dmz_objects[i]->status == DEAD) continue;

		sprintf(buf, "%s          %d", dmz_objects[i]->name, dmz_objects[i]->u.player.frags);

		DrawString(buf, SCORE_X, SCORE_Y + (players * 20));

		players++;
		if (players > 20) break;
	}

	(i == MAXOBJECTS) ? (last = 0) : (last = i);
}

void DrawString(char *string, int x, int y)
{
	int i;

	for (i = 0; string[i] != '\0'; i++)
	{
		if (string[i] == ' ') continue;

		else if (string[i] >= '0' && string[i] <= '9')
			dmz_graph_virblit(numbers, 0, (string[i] - '0') * FONT_W, 0, FONT_W, FONT_H,
				x + (i * FONT_W), y, TRANSPARENT_COLOR);
		else
			dmz_graph_virblit(alphabet, 0, (string[i] - 'A') * FONT_W, 0, FONT_W, FONT_H,
				x + (i * FONT_W), y, TRANSPARENT_COLOR);
	}
}

void DrawPlayerRadarBlip(int player, unsigned int color)
{
	int my_x, my_y, other_x, other_y, dif_x, dif_y;

	my_x = dmz_objects[my_id]->x;
	my_y = dmz_objects[my_id]->y;
	other_x = dmz_objects[player]->x;
	other_y = dmz_objects[player]->y;
	dif_x = (my_x - other_x) / PIXELS_PER_CELL;
	dif_y = (my_y - other_y) / PIXELS_PER_CELL;

	/*	sprintf(buf, "object %d diff (%d, %d)", i, dif_x, dif_y);
		dmz_logstd(buf);
	*/
	if (dif_x >= -50 && dif_x <= 50 &&
		dif_y >= -50 && dif_y <= 50)
	{
		dmz_graph_setpixel(0, (721 - dif_x), (474 - dif_y), color);
		dmz_graph_setpixel(0, (720 - dif_x), (474 - dif_y), color);
		dmz_graph_setpixel(0, (722 - dif_x), (474 - dif_y), color);
		dmz_graph_setpixel(0, (721 - dif_x), (473 - dif_y), color);
		dmz_graph_setpixel(0, (721 - dif_x), (475 - dif_y), color);
	}
}

void DrawOtherRadarBlip(int player, int width, int height, unsigned int color)
{
	int my_x, my_y, other_x, other_y, dif_x, dif_y;
	int i, j;

	my_x = dmz_objects[my_id]->x;
	my_y = dmz_objects[my_id]->y;
	other_x = dmz_objects[player]->x;
	other_y = dmz_objects[player]->y;
	dif_x = (my_x - other_x) / PIXELS_PER_CELL;
	dif_y = (my_y - other_y) / PIXELS_PER_CELL;

	/*	sprintf(buf, "object %d diff (%d, %d)", i, dif_x, dif_y);
		dmz_logstd(buf);
	*/
	if (dif_x >= -50 && dif_x <= 50 &&
		dif_y >= -50 && dif_y <= 50)
	{
		for (i = 0; i < width; i++)
			for (j = 0; j < height; j++)
			{
				dmz_graph_setpixel(0, (721 + i - dif_x), (474 + j - dif_y), color);
			}
	}
}

void DrawProgressIndicator(int percent, int flag)
{
	dmz_graph_blit(splash, 0, 0, 800, 600, 0, 0, -1);
	if (flag) dmz_graph_blit(splash2, 0, 0, 320, 32, 240, 440, -1);
	dmz_graph_blit(progress, 0, 0, percent * 6, 50, 100, 500, -1);
	dmz_graph_flip();
}
