/*	File:	local.c
	Author:	Alex Fedosov, Emory Al-Imam
	Date:	March 15, 2001
	Description:	functions related to local player
*/

#include <stdio.h>

#include "client.h"
#include "input.h"

#include "dmz_net.h"
#include "dmz_snd.h"
#include "dmzcore.h"

GLOBAL short leftMouseDown;
GLOBAL short rightMouseDown;
GLOBAL short midMouseDown;

GLOBAL int snd_fire, snd_hth, snd_empty;

GLOBAL int hth;

/* move in a direction, where dx and dy can be [-1,0,1]
   e.g. if moving left, dx would be -1 and dy would be 0 */
int do_move(int directionX, int directionY)
{
	struct dmz_message m;
	int dir = dmz_delta2dir(directionX, directionY);
	int velocity = dmz_objects[my_id]->velocity;
	WORD32 new_x, new_y;
	WORD32 old_x, old_y;
	WORD16 colid;
	CELL cell;
	short dam, spd;

	if (directionX == 0 && directionY == 0)
		return 0;

	/* update direction */
	dmz_objects[my_id]->u.player.direction = dir;
	dmz_objects[my_id]->status = MOVING;

	old_x = dmz_objects[my_id]->x;
	old_y = dmz_objects[my_id]->y;

	/* update player's position */
	new_x = dmz_objects[my_id]->x += velocity * directionX;
	new_y = dmz_objects[my_id]->y += velocity * directionY;

	/* check if out of bounds */
	if (new_x > map_width * PIXELS_PER_CELL ||
		new_y > map_height * PIXELS_PER_CELL)
	{
		/* undo the move */
		dmz_objects[my_id]->x = old_x;
		dmz_objects[my_id]->y = old_y;
		return 0;
	}

	if (!ValidMove(dir, my_id))
	{
		/* undo the move */
		dmz_objects[my_id]->x = old_x;
		dmz_objects[my_id]->y = old_y;
		return 0;
	}

	colid = DetectCollision((WORD16) my_id);
	if (colid > 0)
	{
		sprintf(buf, "collided with %d type %d", colid, dmz_objects[colid]->objtype);
		dmz_logstd(buf);

		/* we're allowed to run over small terrain objects */
		if (dmz_objects[colid]->objtype == TERRAIN &&
			dmz_objects[colid]->width < 64)
		{
				/* kill that thing */
				dmz_status_update(colid, DEAD);
		}

		/* the only allowed collision is with items */
		else if (dmz_objects[colid]->objtype == ITEM)
		{
			/* if ran over booty, delete it */
			if (dmz_objects[colid]->u.item.kind == BOOTY)
				dmz_delete_object(colid);

			/* others updated by server */			
		}
/*
		else if (dmz_objects[colid]->objtype == NPC)
		{
			NPC_action(colid, my_id);
		}
*/
		else
		{
			/* undo the move */
			dmz_objects[my_id]->x = old_x;
			dmz_objects[my_id]->y = old_y;
			return 0;
		}
	}

	/* apply damage and speed change, if applicable */
	cell = dmz_world_map[new_x / PIXELS_PER_CELL][new_y / PIXELS_PER_CELL];
	dmz_map_get_prop(cell, NULL, &dam, &spd);

	dmz_objects[my_id]->velocity = DEFAULT_VELOCITY + spd;

	/* inform the server of new coordinates */
	memset(&m, 0, sizeof(struct dmz_message));
	m.type = MOVE;
	m.objid = my_id;
	m.u.move.new_x = new_x;
	m.u.move.new_y = new_y;
	m.u.move.direction = dir;

	dmz_send_msg(&m, sizeof(struct dmz_message),  connid);

	/* scroll the screen */
	set_screen_pos(new_x, new_y);

	return 1;
}

void do_chat(char *buffer)
{
	struct dmz_message m;
	int rv;

	/* easter egg */
	if (strcmp(buffer, "ALL YOUR BASE ARE BELONG TO US") == 0)
	{
		dmz_snd_unload(snd_bg);
		snd_bg = dmz_snd_load("data/Sound/aybabtu.wav");
		dmz_snd_set_volume(snd_bg, 70);
		dmz_snd_play(snd_bg, DMZ_SND_LOOP);
	}

	/* another easter egg */
	if (strcmp(buffer, "BABEL") == 0)
	{
		sprintf(buffer, "DONT TALK ABOUT BABEL");
	}

	memset(&m, 0, sizeof(struct dmz_message));

	m.type = CHAT;
	m.objid = my_id;

	/* copy chat buffer into message */
	memcpy(&(m.u.chat.text), buffer, MAXCHATBUF);
			
	rv = dmz_send_msg(&m, sizeof(struct dmz_message), connid);
	if (rv == DMZ_NET_SEND_ERROR)
		dmz_logerr("[client:client:do_chat] could not send to server");
}

/* translate absolute coordinates of a mouse click into dx, dy
   suitable for passing to doMove().
   note: hard-coded for local player */
int dmz_mouse2delta(int mouse_x, int mouse_y, int *dx, int *dy)
{
	int x_dif = mouse_x - (dmz_objects[my_id]->x);
	int y_dif = mouse_y - (dmz_objects[my_id]->y);
	int pos_lim = (DEFAULT_HEIGHT >> 1);
	int neg_lim = 0 - (DEFAULT_HEIGHT >> 1);

	dmz_assert(dx);
	dmz_assert(dy);

	/* adjust for movement in a straight line */
	if (x_dif > 0) x_dif -= pos_lim;
	else x_dif += pos_lim;
	if (y_dif > 0) y_dif -= pos_lim;
	else y_dif += neg_lim;

 	if (x_dif < pos_lim && x_dif > neg_lim) x_dif = 0;
	if (y_dif < pos_lim && y_dif > neg_lim) y_dif = 0;

	*dx = x_dif;
	*dy = y_dif;

	if (x_dif < 0) *dx = -1;
	else if (x_dif > 0) *dx = 1;

	if (y_dif < 0) *dy = -1;
	else if (y_dif > 0) *dy = 1;

	return dmz_delta2dir(*dx, *dy);
}

void ProcessLocalInput(void)
{
	int dx = 0, dy = 0;
	int mouse_x, mouse_y;

	dmz_get_keyboard_input(&dx, &dy);

	/* first, check if we need to exit */
	if (show_exit && leftMouseDown)
	{
		dmz_get_cursor_loc(&mouse_x, &mouse_y);
		
		/* yes, quit */
		if(mouse_x >= 244 && mouse_x <= 304 &&
		   mouse_y >= 293 && mouse_y <= 314)
		{
			exit_game = 1;
		}

		/* no, don't quit */
		else if(mouse_x >= 333 && mouse_x <= 393 &&
		   	    mouse_y >= 293 && mouse_y <= 314)
		{
			show_exit = 0;
		}
	}

	if (dmz_objects[my_id]->status == DEAD)
	{
		set_screen_pos(pixx + (dx + (DELTAX >> 1)) * PIXELS_PER_CELL,
			pixy + (dy + (DELTAY >> 1)) * PIXELS_PER_CELL);
	}

	else
	{
		static int abs_mx = 0;
		static int abs_my = 0;
		static int stillMovingFromMouse = 0;
		WORD16 dir = dmz_objects[my_id]->u.player.direction;

		if (dx != 0 || dy != 0)	/* keyboard movement */
		{
			stillMovingFromMouse = 0; /* cancel mouse movement */
			do_move(dx, dy);
		}

		else if (leftMouseDown == 1) /* mouse clicked */
		{
			dmz_get_cursor_loc(&mouse_x, &mouse_y);

			/* calculate absolute mouse position */
			abs_mx = pixx + mouse_x;
			abs_my = pixy + mouse_y;
			
			dmz_mouse2delta(abs_mx, abs_my, &dx, &dy);
			stillMovingFromMouse = do_move(dx, dy);
		}

		else if (stillMovingFromMouse) /* continue previous mouse */
		{
			dmz_mouse2delta(abs_mx, abs_my, &dx, &dy);
			stillMovingFromMouse = do_move(dx, dy);
		}

		/* projectile attack */
		if ((sgdwFrame % 5) == 0 && (kb[VK_CONTROL].down == 1 ||
			rightMouseDown == 1))
		{
			/* change direction facing if mouse clicked */
			if (rightMouseDown == 1)
			{
				int mouse_x, mouse_y, dx, dy;
				dmz_get_cursor_loc(&mouse_x, &mouse_y);
		
				dir = dmz_mouse2delta(pixx + mouse_x, pixy + mouse_y,
					&dx, &dy);

				/*dmz_objects[my_id]->u.player.direction = dir;*/
			}

			do_attack(my_id, PROJECTILE, dir);
		}

		/* hand-to-hand attack */
		if ((sgdwFrame % 5) == 0 && (kb[VK_NUMPAD0].down == 1 ||
			midMouseDown == 1 || kb['W'].down == 1))
		{
			dmz_snd_set_volume (snd_hth, 85);
			dmz_snd_play(snd_hth, 0);

			do_attack(my_id, HTH, dir);
		}
	}
}

int do_attack(WORD16 player, WORD16 method, WORD16 dir)
{
	struct dmz_message m;
	memset(&m, 0, sizeof(struct dmz_message));

	m.objid = player;
	m.type = ATTACK;
	m.u.attack.direction = dir;
	m.u.attack.method = method;

	if (method == PROJECTILE)
	{
		if( dmz_objects[my_id]->u.player.ammo > 0)
		{
			dmz_snd_play(snd_fire, 0);
			MissileAttack((WORD16) my_id, dir);
		}

		else
		{
			dmz_snd_play(snd_empty, 0);
		}
	}

	else if (method == HTH)
	{
		HTHAttack(player, dmz_objects[player]->u.player.direction);
	}

	dmz_send_msg(&m, sizeof(struct dmz_message), connid);

	return 0;
}

void HTHAttack(WORD16 attacker, WORD direction)
{
	WORD16 shock;
	WORD32 x = dmz_objects[attacker]->x - 4;
	WORD32 y = dmz_objects[attacker]->y + 8;

	/* hand-to-hand attack, fake missile */
	shock = dmz_new_object(0, MISSILE, x, y);
	strcpy(dmz_objects[shock]->name, "hth");

	dmz_objects[shock]->status = DEAD;
	dmz_objects[shock]->u.missile.direction = direction;
	dmz_objects[shock]->u.missile.damage = 0;
	dmz_objects[shock]->u.missile.owner = attacker;
	dmz_objects[shock]->u.missile.exp_ttl = EXPLOSION_TTL;
	dmz_objects[shock]->bitidx = hth;
	dmz_objects[shock]->deadbit = hth;
	dmz_objects[shock]->width = 48;		// TODO dont hardcode
	dmz_objects[shock]->height = 48;
}
