/*	File:	winserv.c
	Author:	Alex Fedosov
	Date:	December 12, 2000
	Description:	server prototype -- windows version
*/

#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <stdio.h>
#include <time.h>

#include "resource.h"

#include "sys/lock.h"
#include "sys/thread.h"

#include "server.h"

#include "dmzcore.h"
#include "dmzgraph.h"
#include "dmz_net.h"

//#define DELTAX 20
//#define DELTAY 15

LRESULT CALLBACK WndProc(HWND hWnd, unsigned uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK ServerDialogProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK AboutDialogProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
void UpdateServerDialog(HWND hWnd);
void ServerDraw(HWND hWnd);
int GameProcess(void);
void ProcessUnits(void);

HINSTANCE hInstance;
HWND hWnd;
HBITMAP map;
static char szClass[] = "Ryll Server";
int combo_contents[MAXCLIENTS];
char buf[256];

int sgdwFrame = 0;
int X = 0, Y = 0;
BOOL sgfAppActive = FALSE;
time_t start_time, finish_time;

/* variables for server information */
char my_ip[16];

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	BOOL fRunning = TRUE;
	MSG	tMsg;
	WNDCLASSEX wc;

	int rv;
	unsigned int h, tid;
	__lock_t mylock;

	/* set up and register window class */
	wc.cbSize = sizeof(WNDCLASSEX);

	wc.hInstance = hInstance;
	wc.lpszClassName = szClass;
	wc.lpfnWndProc = (WNDPROC) ServerDialogProc;
	wc.style = CS_HREDRAW | CS_VREDRAW;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wc.hIconSm = LoadIcon(NULL, IDI_WINLOGO);
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.lpszMenuName = NULL;
	wc.hbrBackground = (HBRUSH) GetStockObject(GRAY_BRUSH);

	if (!RegisterClassEx(&wc))
	{
		MessageBox(NULL, "RegisterClass() failed", "Error!", MB_OK | MB_ICONERROR);
		return FALSE;
	}

	/* create a window and display it */
/*	hwnd = CreateWindow(szClass, "Ryll Server", WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
		HWND_DESKTOP, NULL, hInstance, NULL);

	if (!hwnd)
	{
		MessageBox(NULL, "CreateWindow() failed", "Error!", MB_OK | MB_ICONERROR);
		return FALSE;
	}
*/
	dmz_logstd("[server:winserv:WinMain] starting intialization");

	/* initialize graphics library */
	rv = dmz_graph_init(NULL);

	/* create the I/O thread */
	dmz_logstd("[server:winserv:WinMain] spawning I/O thread...");

	__lock_init(mylock);
	__mutex_lock(mylock);

	h = dmz_thread_create(dmz_server_main, &mylock, &tid);
	if (h == 0)
	{
		MessageBox(NULL, "ThreadCreate() failed", "Error!", MB_OK | MB_ICONERROR);
		return FALSE;
	}

	/* grab the lock again for mutual exclusion */
	dmz_logstd("[server:winserv:WinMain] waiting for lock release...");

	__mutex_lock(mylock);
	__mutex_unlock(mylock);

	dmz_logstd("[server:winserv:WinMain] lock released");
	
	map = (HBITMAP) LoadImage(NULL, "dmz_map.bmp", IMAGE_BITMAP, 0, 0,
		LR_DEFAULTSIZE | LR_LOADFROMFILE);

//	ShowWindow(hwnd, nCmdShow);
//	UpdateWindow(hwnd);

	sgfAppActive = TRUE;
	start_time = time(NULL);

	hWnd = CreateDialog(hInstance, "IDD_SERVER_DIALOG", HWND_DESKTOP, (DLGPROC) ServerDialogProc);

	/* run until game says stop */
	while (fRunning)
	{
		if (PeekMessage(&tMsg, NULL, 0, 0, PM_NOREMOVE))
		{
			if (!GetMessage(&tMsg, NULL, 0, 0L))
				fRunning = FALSE;

			TranslateMessage(&tMsg);
			DispatchMessage(&tMsg);
		}

		else if (!sgfAppActive)
		{
			WaitMessage();
		}

		else
		{
			if (!ServerProcess())
				DestroyWindow(hWnd);
		}
	}

	return tMsg.wParam;
}

BOOL CALLBACK ServerDialogProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	HWND hDlgItem;
	char buffer[100];
	char ip[16] = "";
	static int selected_client = 0;

	switch (uMsg)
	{
	case WM_INITDIALOG:
		hDlgItem = GetDlgItem(hWnd, IDC_DISCONNECT);
		EnableWindow(hDlgItem, 0);
		SetTimer(hWnd, 1, 1000, NULL);
		UpdateServerDialog(hWnd);
		dmz_net_get_local_IP(ip);
		sprintf(buffer, "Return to Ryll Server on %s", ip);
		SendMessage (hWnd, WM_SETTEXT, 0, (LPARAM) buffer);
		break;

	case WM_DESTROY:
		dmz_server_kill();
		exit(0);
		break;

	case WM_KEYDOWN:
		if (wParam == VK_ESCAPE)
			DestroyWindow(hWnd);
		break;

	case WM_TIMER:
		hDlgItem = GetDlgItem(hWnd, IDC_MAP);
		UpdateServerDialog(hWnd);
		InvalidateRect(hDlgItem, NULL, TRUE);
		ServerDraw(hDlgItem);
		break;

	case WM_PAINT:
		ServerDraw(GetDlgItem(hWnd, IDC_MAP));
		break;

	case WM_COMMAND:

		switch (HIWORD(wParam))
		{
			case CBN_SELENDOK:
			{
				int n, conidx, objid;
				n = SendDlgItemMessage(hWnd, IDC_CLIENT_COMBO, CB_GETCURSEL, 0, 0);

				sprintf(buf, "cursor selection %d", n);
				dmz_logstd(buf);

				if (n >= 0)
				{
					conidx = combo_contents[n];
					objid = clients[conidx].objid;

					SendDlgItemMessage(hWnd, IDC_EDIT_IP, WM_SETTEXT, 0, (LPARAM) clients[conidx].ip_addr);
					SendDlgItemMessage(hWnd, IDC_EDIT_NAME, WM_SETTEXT, 0, (LPARAM) dmz_objects[objid]->name);

					sprintf(buffer, "%d", dmz_objects[objid]->score);
					SendDlgItemMessage(hWnd, IDC_EDIT_SCORE, WM_SETTEXT, 0, (LPARAM) buffer);

					sprintf(buffer, "%d", dmz_objects[objid]->u.player.life);
					SendDlgItemMessage(hWnd, IDC_EDIT_LIFE, WM_SETTEXT, 0, (LPARAM) buffer);

					sprintf(buffer, "%d", dmz_objects[objid]->u.player.ammo);
					SendDlgItemMessage(hWnd, IDC_EDIT_AMMO, WM_SETTEXT, 0, (LPARAM) buffer);

					/*hDlgItem = GetDlgItem(hWnd, IDC_DISCONNECT);
					EnableWindow(hDlgItem, 1);*/

					selected_client = conidx;
				}
			}
			break;
		}

		switch (LOWORD(wParam))
		{
		
			case IDC_DISCONNECT:		/* disconnect selected player */
				break;

			case ID_FILE_EXIT:
			//case IDOK:
			case IDC_EXIT:				/* shut down the server */
				DestroyWindow(hWnd);
				break;

			case ID_HELP_ABOUT:
			case IDC_ABOUT:
				//MessageBox(hWnd, "Return to Ryll Server\nTeam Overclocked 2001", "About", MB_OK);
				//hWnd = 
					CreateDialog(hInstance, "IDD_ABOUT_DIALOG", hWnd, (DLGPROC) AboutDialogProc);
				break;
		}
		break;
	}

	return 0;
}

BOOL CALLBACK AboutDialogProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	HWND hBases;
	static int status = FALSE;

	switch (uMsg)
	{
		case WM_INITDIALOG:
			SetTimer(hWnd, 1, 5000, NULL);
			break;

		case WM_COMMAND:
			{
				switch (LOWORD(wParam))
				{
				case IDOK:
					DestroyWindow(hWnd);
					break;
				}
			}
			break;

		case WM_TIMER:
			switch (wParam)
			{
			case 0:
				hBases = GetDlgItem(hWnd, IDC_CATS);
				status = !status;
				ShowWindow(hBases, status);
				break;
			case 1:
				KillTimer(hWnd, 1);
				SetTimer(hWnd, 0, 500, NULL);
				break;
			}
			break;
	}
	return 0;
}

void UpdateServerDialog(HWND hWnd)
{
	time_t uptime, hrs, mins, secs;
	int numclients, servers, players, i;
	char buffer[32];

	for (i = 0; i < MAXCLIENTS; i++)
	{
		combo_contents[i] = -1;
	}
	
	/* display uptime */
	uptime = time(NULL) - start_time;
	hrs = uptime / 3600;
	mins = (uptime % 3600) / 60;
	secs = (uptime % 3600) % 60;
	sprintf(buffer, "%02d:%02d:%02d", hrs, mins, secs);
	SendDlgItemMessage(hWnd, IDC_EDIT_UPTIME, WM_SETTEXT, 0, (LPARAM) buffer);

	/* display number of clients */
	numclients = dmz_get_num_clients();
	sprintf(buffer, "%d", numclients);
	SendDlgItemMessage(hWnd, IDC_EDIT_CLIENTS, WM_SETTEXT, 0, (LPARAM) buffer);

	/* display number of servers */
	servers = dmz_get_num_servers();
	sprintf(buffer, "%d", servers);
	SendDlgItemMessage(hWnd, IDC_EDIT_SERVERS, WM_SETTEXT, 0, (LPARAM) buffer);

	/* display number of players */
	players = dmz_get_num_players();
	sprintf(buffer, "%d", players);
	SendDlgItemMessage(hWnd, IDC_EDIT_PLAYERS, WM_SETTEXT, 0, (LPARAM) buffer);

	SendDlgItemMessage(hWnd, IDC_CLIENT_COMBO, CB_RESETCONTENT, 0, 0);
	for (i = 0; i < MAXCLIENTS; i++)
	{
		if (clients[i].connid > 0)
		{
			int objid = clients[i].objid;
			char *name = dmz_objects[objid]->name;
			int n;

			n = SendDlgItemMessage(hWnd, IDC_CLIENT_COMBO, CB_ADDSTRING, 0, (LPARAM) name);

			//sprintf(buf, "added selection at %d", n);
			//dmz_logstd(buf);

			combo_contents[n] = i;
		}
	}
}

void ServerDraw(HWND hWnd)
{
	HBITMAP hold, hBit = map;
	HDC hdc, memdc;
	RECT rect;
	BITMAP b;
	COLORREF color;
	int i, x, y;

	GetObject(hBit, sizeof(BITMAP), &b);
	GetClientRect(hWnd, &rect);

	hdc = GetDC(hWnd);
	memdc = CreateCompatibleDC(hdc);
	hold = SelectObject(memdc, hBit);

	StretchBlt(hdc, rect.left, rect.top,
		(rect.right - rect.left), (rect.bottom - rect.top),
		memdc, 0, 0, b.bmWidth, b.bmHeight, SRCCOPY);

	for (i = 0; i < MAXOBJECTS; i++)
	{
		if (dmz_objects[i] == NULL) continue;
		if (dmz_objects[i]->status == DEAD || dmz_objects[i]->status == INVISIBLE ||
			dmz_objects[i]->status == SLEEPING) continue;

		x = dmz_objects[i]->x / PIXELS_PER_CELL;
		y = dmz_objects[i]->y / PIXELS_PER_CELL;

		switch (dmz_objects[i]->objtype)
		{
		case PLAYER:
		case MONSTER:
			color = RGB(255, 0, 0);
			SetPixel(hdc, x, y, color);
			SetPixel(hdc, x - 1, y, color);
			SetPixel(hdc, x + 1, y, color);
			SetPixel(hdc, x, y - 1, color);
			SetPixel(hdc, x, y + 1, color);
			break;
		case ITEM:
			color = RGB(255, 255, 255);
			SetPixel(hdc, x, y, color);
			SetPixel(hdc, x - 1, y, color);
			SetPixel(hdc, x + 1, y, color);
			SetPixel(hdc, x, y - 1, color);
			SetPixel(hdc, x, y + 1, color);
			break;
		case TERRAIN:
			color = RGB(128, 0, 0);
			SetPixel(hdc, x, y, color);
			SetPixel(hdc, x - 1, y, color);
			SetPixel(hdc, x + 1, y, color);
			SetPixel(hdc, x, y - 1, color);
			SetPixel(hdc, x, y + 1, color);
			break;
		default:
			break;
		}
	}
	
	SelectObject(memdc, hold);
	ReleaseDC(hWnd, hdc);
	DeleteDC(memdc);
}

int ServerProcess(void)
{
	DWORD   dwTick = GetTickCount();
	static DWORD sgdwNextFrame = 0;

	/* check for frame advance */
	if (dwTick >= sgdwNextFrame)
	{
		sgdwNextFrame = dwTick + 40;	/* set next frame */
		sgdwFrame++;

		/* Process units no more than 25 times per second */
		ProcessUnits();
	}

	return (TRUE);
}

void ProcessUnits(void)
{
	int state = 0;
	int dx = 0, dy = 0;
	WORD16 i;

	/* go through all the objects and animate as needed */
	for (i = 0; i < MAXOBJECTS; i++)
	{
		struct dmz_object *o = dmz_objects[i];
		if (dmz_objects[i] == NULL) continue;

		if (dmz_objects[i]->status == DEAD ||
			dmz_objects[i]->status == INVISIBLE)
		{
			if ((sgdwFrame % 125) == 0)
				Regenerate(i);

			continue;
		}

		switch (dmz_objects[i]->objtype)
		{
		case MONSTER:
			ApplyAI(i);
			break;
		case MISSILE:
			MoveMissile(i);
			break;

		default:	/* don't animate anything else */
			break;
		}
	}
}

void dmz_server_kill(void)
{
	finish_time = time(NULL);

	sprintf(buf, "Frame Rate: %.2f fps", (float) sgdwFrame / (float) (finish_time - start_time));
	dmz_logstd(buf);

	dmz_object_destroy();
	dmz_logstd("[client:client:dmz_client_kill] object destruction");
	dmz_map_destroy();
	dmz_logstd("[client:client:dmz_client_kill] map destruction");
	dmz_graph_kill();
	dmz_logstd("[client:client:dmz_client_kill] graphics shutdown");
	dmz_net_kill();
	dmz_logstd("[client:client:dmz_client_kill] network shutdown");
}
